﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Termination {
  [Item("ExecutionTimeTerminationCriterion", "A termination criterion based on execution time of an algorithm.")]
  [StorableClass]
  public class ExecutionTimeTerminationCriterion : ThresholdTerminationCriterion<TimeSpanValue> {

    [Storable]
    private readonly IExecutable executable;

    [StorableConstructor]
    protected ExecutionTimeTerminationCriterion(bool deserializing) : base(deserializing) { }
    protected ExecutionTimeTerminationCriterion(ExecutionTimeTerminationCriterion original, Cloner cloner)
      : base(original, cloner) {
      executable = cloner.Clone(executable);
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new ExecutionTimeTerminationCriterion(this, cloner);
    }
    public ExecutionTimeTerminationCriterion(IExecutable executable)
      : this(executable, new TimeSpan(0, 10, 0)) { }
    public ExecutionTimeTerminationCriterion(IExecutable executable, TimeSpan maximumExecutionTime) {
      this.executable = executable;
      ThresholdParameter.Value = new TimeSpanValue(maximumExecutionTime);

      Name = "Maximum Execution Time";
    }

    protected override bool CheckTermination() {
      var max = ThresholdParameter.Value.Value;

      return executable.ExecutionTime > max;
    }
  }
}